unit NDesktop;

{$IMPORTEDDATA OFF}

interface

uses NGraphics;

procedure Open(const Args: array of WideString; const Dir: WideString='');
procedure LoadFileIcon(const Icon: TIcon; const Path: WideString; Index: Integer=-1);
procedure OpenDir(const Path: WideString);

implementation

uses Windows, ShellAPI, NLib, NSys;

// fix ShellAPI unit declaration bug
function SHGetFileInfoW(pszPath: PWideChar; dwFileAttributes: DWORD;
    var sfi: TSHFileInfoW; cbFileInfo, uFlags: UINT): DWORD;
    stdcall external shell32;

procedure Open(const Args: array of WideString; const Dir: WideString='');
var
  i: Integer;
  base, path: WideString;
begin
  base := BaseName(Args[0]);
  // XP needs '\'
  path := Args[0]; // WideString always copyed
  for i := 1 to Length(path) do
    if path[i] = '/' then path[i] := '\';
  if (base = '.') or (base = '..') then
    path := path +  '\';
  if ShellExecuteW(GetDesktopWindow, nil, PWideChar(Path),
      PWideChar(JoinCommandLine(SliceArrayW(Args, 1))), PWideChar(Dir),
      SW_SHOWNORMAL) <= 32 then
    raise EOSError.Create;
end;

// must call CoInitialize
procedure LoadFileIcon(const Icon: TIcon; const Path: WideString; Index: Integer=-1);
var
  sfi: TSHFileInfoW;
begin
  if Icon.Handle <> 0 then
    DestroyIcon(Icon.Handle);
  if Index > -1 then
    Icon.Handle := ExtractIconW(HInstance, PWideChar(Path), Index)
  else if SHGetFileInfoW(PWideChar(Path), 0, sfi, SizeOf(sfi), SHGFI_ICON) <> 0 then
    Icon.Handle := sfi.hIcon
  else
    Icon.Handle := 0;
end;

procedure OpenDir(const Path: WideString);
var
  h: HMODULE;
  SHParseDisplayName: function (pszName: PWideChar; pdc: Pointer;
      var pidl: Pointer; sfgaoIn: ULONG; psfgaoOut: PULONG): HRESULT; stdcall;
  SHOpenFolderAndSelectItems: function (pidlFolder: Pointer; cidl: UINT;
      apidl: Pointer; dwFlags: DWORD): HRESULT; stdcall;
  ILFree: procedure (pidl: Pointer); stdcall;
  pidl: Pointer;
  len: UINT;
  windir: WideString;
begin
  h := GetModuleHandle(shell32);
  if h <> 0 then begin
    SHParseDisplayName := GetProcAddress(h, 'SHParseDisplayName');
    SHOpenFolderAndSelectItems := GetProcAddress(h, 'SHOpenFolderAndSelectItems');
    ILFree := GetProcAddress(h, 'ILFree');
    if Assigned(SHParseDisplayName) and Assigned(SHOpenFolderAndSelectItems)
        and Assigned(ILFree) then begin
      // XP and later
      // must call CoInitialize
      if SHParseDisplayName(PWideChar(Path), nil, pidl, 0, nil) <> S_OK then
        raise EOSError.Create;
      try
        if SHOpenFolderAndSelectItems(pidl, 0, nil, 0) <> S_OK then
          raise EOSError.Create;
      finally
        ILFree(pidl); // instead of CoTaskMemFree to avoid using ActiveX unit
      end;
      Exit;
    end;
  end;

  SetString(windir, nil, MAX_PATH);
  len := GetWindowsDirectoryW(@windir[1], Length(windir));
  if len = 0 then
    raise EOSError.Create;
  SetLength(windir, len);
  if ShellExecuteW(GetDesktopWindow, 'open',
      PWideChar(windir + '\explorer.exe'), PWideChar('/select,"' + Path + '"'),
      nil, SW_SHOWNORMAL) <= 32 then
    raise EOSError.Create;
end;

end.
